import datetime
import json

from databases import Database
import sqlalchemy

from .base import BaseRecorder
from ..op import OpDetails

metadata = sqlalchemy.MetaData()

events_id_seq = sqlalchemy.Sequence('events_id_seq', metadata=metadata)

events = sqlalchemy.Table(
    'events', metadata,
    sqlalchemy.Column('id', sqlalchemy.Integer, events_id_seq, primary_key=True),
    sqlalchemy.Column('time', sqlalchemy.Time),
    sqlalchemy.Column('name', sqlalchemy.Text),
    sqlalchemy.Column('payload', sqlalchemy.Text),
    sqlalchemy.Column('unknown', sqlalchemy.Boolean),
)

packets_id_seq = sqlalchemy.Sequence('packets_id_seq', metadata=metadata)

packets = sqlalchemy.Table(
    'packets', metadata,
    sqlalchemy.Column('id', sqlalchemy.Integer, packets_id_seq, primary_key=True),
    sqlalchemy.Column('time', sqlalchemy.Time),
    sqlalchemy.Column('op_code', sqlalchemy.Integer),
    sqlalchemy.Column('event_name', sqlalchemy.Text),
    sqlalchemy.Column('payload', sqlalchemy.Text),
    sqlalchemy.Column('inbound', sqlalchemy.Boolean),
)

class DatabasesRecorder(BaseRecorder):
    started: bool = False
    def __init__(self, database_url: str):
        self.db: Database = Database(database_url)
    
    async def start(self) -> None:
        await self.db.connect()
        self.started = True
    
    async def end(self) -> None:
        await self.db.disconnect()
        self.started = False
    
    async def last_events_id(self) -> int:
        query = packets.select().order_by(packets.c.id.desc()).limit(1)
        packets_id = await self.db.fetch_val(query)
        return packets_id

    async def save_events(self, name: str, payload: dict, *, unknown: bool=False) -> None:
        query = events.insert().values(
            time=datetime.datetime.utcnow(),
            name=name,
            payload=json.dumps(payload),
            unknown=unknown,
        )
        await self.db.execute(query)
    
    async def save_packets(self, op_code: int, details: OpDetails) -> None:
        query = packets.insert().values(
            time=datetime.datetime.utcnow(),
            op_code=op_code,
            event_name=details.event_name,
            payload=json.dumps(details.payload),
            inbound=details.inbound,
        )
        await self.db.execute(query)

    async def save_requests(self, *args, **kwargs) -> None:
        return self._save_requests(*args, **kwargs)


import typing
from dataclasses import dataclass

from .flags import EventFlags, OpFlags
from .gateway import SpaceShuttle
from .dawn import Earth
from .recorders.base import BaseRecorder
from .http import Meteor, Pitch

@dataclass
class Configuration:
    recorder: BaseRecorder
    event_flags: EventFlags = EventFlags.all()
    op_flags: OpFlags = OpFlags.all()
    gw_cls: SpaceShuttle = SpaceShuttle
    anal_cls: Earth = Earth
    httpclient_cls: Pitch = Pitch
    session_cls: Meteor = Meteor
    
Config = Configuration


from distutils.core import setup
    
version = '0.1.0'

with open('README.md', encoding='utf-8') as f:
    readme = f.read()

setup(
    name='nextcord-ext-space',
    author='japandotorg',
    python_requires='>=3.8.0',
    url='https://github.com/japandotorg/nextcord-ext-space',
    version=version,
    packages=[
        'nextcord/ext/space',
        'nextcord/ext/space/recorders',
    ],
    license='MIT',
    description='A event logger for nextcord discord API library.',
    long_description=readme,
    long_description_content_type='text/markdown',
    include_package_data=True,
    install_requires=[
        'nextcord==2.0.0a6',
        'databases',
    ],
    keywords=[
        'nextcord',
        'extension',
        'discord'
    ],
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Internet',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
    ]
)


from ..op import OpDetails

class BaseRecorder:
    started: bool
    async def start(self) -> None: ...
    async def end(self) -> None: ...
    async def last_events_id(self) -> int: ...
    async def save_events(self, name: str, payload: dict, *, unknown: bool) -> None: ...
    async def save_packets(self, op_code: str, details: OpDetails) -> None: ...